@@ -13,17 +13,21 @@ function (B, P, ps, yy, w, lambdax, lambdap, center, by)
     basisX = NULL
     penaltyX = NULL
     Bx = NULL
-    for (i in 1:(length(yy)/nrow(B[[1]]))) {
+    for (k in 1:(length(yy)/nrow(B[[1]]))) {
         if (any(!is.na(by[[1]]))) 
             Bx = rbind(Bx, B[[1]] * by[[1]])
         else Bx = rbind(Bx, B[[1]])
     }
+    if (center) {
+        Bx = cbind(1, Bx)
+        P[[1]] = rbind(0, cbind(0, P[[1]]))
+    }
     nx <- ncol(Bx)
     ny <- ncol(By)
     B1 <- kronecker(Bx, t(rep(1, ny)))
     B2 <- kronecker(t(rep(1, nx)), By)
     basisX <- B1 * B2
-    nb = ncol(basisX)
+    nb = ncol(basisX) - 1 * center
     Px <- kronecker(sqrt(lambdax[1]) * P[[1]], diag(ny))
     Dy <- diff(diag(ny), diff = 2)
     Py <- kronecker(diag(nx), sqrt(lambdap[1]) * Dy)
@@ -50,10 +54,6 @@ function (B, P, ps, yy, w, lambdax, lambdap, center, by)
                 ncol = dim(Pxy)[2])), cbind(matrix(0, nrow = dim(Pxy)[1], 
                 ncol = dim(penaltyX)[2]), Pxy))
         }
-    if (center) {
-        basisX = cbind(1, basisX)
-        penaltyX = rbind(0, cbind(0, penaltyX))
-    }
     zx <- rep(0, nrow(penaltyX))
     zplus <- c(yy, zx)
     Bplus <- rbind(basisX, penaltyX)
@@ -69,10 +69,9 @@ function (B, P, ps, yy, w, lambdax, lambdap, center, by)
             break
         cat(it, nw, "\n")
     }
+    pcoef = matrix(pcoef, nrow = B.size + 1)
     if (center) {
-        basisX = basisX[, -1]
-        intercept = pcoef[1]
-        pcoef = pcoef[-1]
+        intercept = pcoef[, 1]
     }
     else intercept = 0
     my = nrow(B[[1]])
@@ -80,17 +79,37 @@ function (B, P, ps, yy, w, lambdax, lambdap, center, by)
     Z = list()
     Bg = basisX[, 1:nb[1]]
     ag = pcoef[1:nb[1]]
-    Z[[1]] = Bg %*% ag + intercept
+    Z[[1]] = Bg %*% ag
     dim(Z[[1]]) = c(my, np)
     if (length(B) > 1) 
         for (i in 2:length(B)) {
             Bg = NULL
             ag = NULL
-            Bg = basisX[, (sum(nb[1:(i - 1)]) + 1):(sum(nb[1:i]))]
-            ag = pcoef[(sum(nb[1:(i - 1)]) + 1):(sum(nb[1:i]))]
-            Z[[i]] = Bg %*% ag + intercept
+            if (center) {
+                Bx = NULL
+                for (k in 1:(length(yy)/nrow(B[[i]]))) {
+                  if (any(!is.na(by[[i]]))) 
+                    Bx = rbind(Bx, B[[i]] * by[[i]])
+                  else Bx = rbind(Bx, B[[i]])
+                }
+                Bx = cbind(1, Bx)
+                nx <- ncol(Bx)
+                ny <- ncol(By)
+                B1 <- kronecker(Bx, t(rep(1, ny)))
+                B2 <- kronecker(t(rep(1, nx)), By)
+                Bg = B1 * B2
+                ag = matrix(pcoef[(sum(nb[1:(i - 1)]) + 1):(sum(nb[1:i]))], 
+                  nrow = B.size + 1)
+                ag = cbind(intercept, ag)
+                ag = as.vector(ag)
+            }
+            else {
+                Bg = basisX[, (sum(nb[1:(i - 1)]) + 1):(sum(nb[1:i]))]
+                ag = pcoef[(sum(nb[1:(i - 1)]) + 1):(sum(nb[1:i]))]
+            }
+            Z[[i]] = Bg %*% ag
             dim(Z[[i]]) = c(my, np)
         }
-    return(list(coef = matrix(pcoef, ncol = np), fit = pfit, 
-        hatma = model$qr, curves = Z, intercept = intercept))
+    return(list(coef = pcoef, fit = pfit, hatma = model$qr, curves = Z, 
+        intercept = intercept))
 }
